import logging
import ftplib
import time

from parallels.operability_check import DomainIssue
from parallels.operability_check import Severity, FTPAuthIssueType

logger = logging.getLogger(__name__)

problem_messages = {
	FTPAuthIssueType.WEB_IP_NOT_DEFINED: 
		u"The domain does not have a web service IP address. FTP access verification will be skipped.",
	FTPAuthIssueType.CHECK_FAILED:
		u"Failed to log in over FTP to the web server at %s as the user %s: %s",
	FTPAuthIssueType.INTERNAL_ERROR:
		u"An internal error occurred while checking if it is possible to log in over FTP to the web server at %s as the user %s: %s"
}

class FTPAuthChecker(object):
	def check(self, domains_to_check):
		issues = []
		for domain_to_check in domains_to_check:
			issues += self._check_single_domain(domain_to_check)
		return issues

	def _check_single_domain(self, domain_to_check):
		issues = []

		def add_issue(severity, category, problem):
			issues.append(DomainIssue(domain_name=domain_to_check.domain_name, severity=severity, category=category, problem=problem))

		if domain_to_check.web_server_ip is None:
			add_issue(
				severity=Severity.WARNING, category=FTPAuthIssueType.WEB_IP_NOT_DEFINED,
				problem=problem_messages[FTPAuthIssueType.WEB_IP_NOT_DEFINED]
			)
			return issues

		for user in domain_to_check.users:
			try:
				self._check_ftp_login(domain_to_check.web_server_ip, user.login, user.password)
			except ftplib.all_errors, e:
				logger.debug(u"Exception:", exc_info=e)
				add_issue(
					severity=Severity.ERROR, category=FTPAuthIssueType.CHECK_FAILED,
					problem=problem_messages[FTPAuthIssueType.CHECK_FAILED] % (
						domain_to_check.web_server_ip, user.login, str(e)
					)
				)
			except KeyboardInterrupt:
				# for compatibility with python 2.4
				raise
			except Exception, e:
				logger.debug(u"Exception:", exc_info=e)
				add_issue(
					severity=Severity.WARNING, category=FTPAuthIssueType.INTERNAL_ERROR,
					problem=problem_messages[FTPAuthIssueType.INTERNAL_ERROR] % (
						domain_to_check.web_server_ip, user.login, str(e)
					)
				)

		return issues

	def _check_ftp_login(self, host, login, password):
		ftp = ftplib.FTP(host)
		ftp.login(login, password)
		ftp.dir(lambda x: None) # just execute a command
		ftp.quit()
		# we sleep 250ms due to ftp server of PPA/Plesk works via xinetd.d that has default limit for 50 connections in 10 seconds interval
		time.sleep(0.25)

